/*
 * Copyright 2011 Red Hat, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *    1. Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *    2. Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in
 *       the documentation and/or other materials provided with the
 *       distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "codec.h"
#include "memory.h"

struct buffer {
  unsigned char *buf;
  unsigned int len;
};

static int
consume_bytes_buffer(const void *buffer, unsigned int len, struct buffer *data)
{
  unsigned char *tmp = realloc(data->buf, data->len + len + 1);
  if (!tmp) {
    free(data->buf);
    data->buf = NULL;
    data->len = 0;
    return -1;
  }

  memcpy(&tmp[data->len], buffer, len);
  tmp[data->len + len] = '\0';

  data->len += len;
  data->buf = tmp;
  return 0;
}

void *
int_ber_decode_(asn_TYPE_descriptor_t *type_descriptor, const char *buf,
    int len)
{
  void *tmp = NULL;

  if (!buf)
    return NULL;

  if (len < 0)
    len = strlen(buf);

  if (ber_decode(NULL, type_descriptor, &tmp, buf, len).code != RC_OK)
    tmp = NULL;

  return tmp;
}

bool
int_der_encode(asn_TYPE_descriptor_t *type_descriptor, void *ptr, char **buf,
    unsigned int *len)
{
  struct buffer zero(buffer);

  if (der_encode(type_descriptor, ptr,
                 (asn_app_consume_bytes_f*) consume_bytes_buffer,
                 &buffer).encoded >= 0) {
    *buf = (char*) buffer.buf;
    *len = buffer.len;
    return true;
  }

  return false;
}

void *
int_uper_decode_(asn_TYPE_descriptor_t *type_descriptor, const char *buf,
                 int len, int skip_bits, int unused_bits)
{
  void *tmp = NULL;

  if (!buf)
    return NULL;

  if (len < 0)
    len = strlen(buf);

  if (uper_decode(NULL, type_descriptor, &tmp, buf,
                  len, skip_bits, unused_bits).code != RC_OK)
    tmp = NULL;

  return tmp;
}

bool
int_uper_encode(asn_TYPE_descriptor_t *type_descriptor, void *ptr, char **buf,
    unsigned int *len)
{
  struct buffer zero(buffer);

  if (uper_encode(type_descriptor, ptr,
                  (asn_app_consume_bytes_f*) consume_bytes_buffer,
                  &buffer).encoded >= 0) {
    *buf = (char*) buffer.buf;
    *len = buffer.len;
    return true;
  }

  return false;
}

void *
int_xer_decode_(asn_TYPE_descriptor_t *type_descriptor, const char *buf,
                int len)
{
  void *tmp = NULL;

  if (!buf)
    return NULL;

  if (len < 0)
    len = strlen(buf);

  if (xer_decode(NULL, type_descriptor, &tmp, buf, len).code != RC_OK)
    tmp = NULL;

  return tmp;
}

bool
int_xer_encode(asn_TYPE_descriptor_t *type_descriptor, void *ptr,
    enum xer_encoder_flags_e xer_flags, char **buf, unsigned int *len)
{
  struct buffer zero(buffer);

  if (xer_encode(type_descriptor, ptr, xer_flags,
                 (asn_app_consume_bytes_f*) consume_bytes_buffer,
                 &buffer).encoded >= 0) {
    *buf = (char*) buffer.buf;
    *len = buffer.len;
    return true;
  }

  return false;
}
